@@ -1,5 +1,6 @@
-from statsmodels.compat.python import (StringIO, urlopen, HTTPError, URLError, lrange,
-                                       cPickle, urljoin, long, PY3)
+from statsmodels.compat.python import (StringIO, urlopen, HTTPError, URLError,
+                                       lrange, urljoin, long, PY3)
+
 import shutil
 from os import environ, makedirs
 from os.path import expanduser, exists, dirname, abspath, join
@@ -114,28 +115,14 @@ def _get_cache(cache):
 
 
 def _cache_it(data, cache_path):
-    if PY3:
-        # for some reason encode("zip") won't work for me in Python 3?
-        import zlib
-        # use protocol 2 so can open with python 2.x if cached in 3.x
-        data = data.decode('utf-8')
-        open(cache_path, "wb").write(zlib.compress(cPickle.dumps(data,
-                                                                 protocol=2)))
-    else:
-        open(cache_path, "wb").write(cPickle.dumps(data).encode("zip"))
+    import zlib
+    open(cache_path, "wb").write(zlib.compress(data))
 
 
 def _open_cache(cache_path):
-    if PY3:
-        # NOTE: don't know why but decode('zip') doesn't work on my
-        # Python 3 build
-        import zlib
-        data = zlib.decompress(open(cache_path, 'rb').read())
-        # return as bytes object encoded in utf-8 for cross-compat of cached
-        data = cPickle.loads(data).encode('utf-8')
-    else:
-        data = open(cache_path, 'rb').read().decode('zip')
-        data = cPickle.loads(data)
+    import zlib
+    data = zlib.decompress(open(cache_path, 'rb').read())
+    # return as bytes object encoded in utf-8 for cross-compat of cached
     return data
 
 
@@ -147,8 +134,14 @@ def _urlopen_cached(url, cache):
     """
     from_cache = False
     if cache is not None:
-        cache_path = join(cache,
-                          url.split("://")[-1].replace('/', ',') + ".zip")
+        file_name = url.split("://")[-1].replace('/', ',')
+        file_name = file_name.split('.')
+        if len(file_name) > 1:
+            file_name[-2] += '-v2'
+        else:
+            file_name[0] += '-v2'
+        file_name = '.'.join(file_name) + ".zip"
+        cache_path = join(cache, file_name)
         try:
             data = _open_cache(cache_path)
             from_cache = True
