@@ -1,26 +1,37 @@
+"""
+Covariance models and estimators for GEE.
+
+Some details for the covariance calculations can be found in the Stata
+docs:
+
+http://www.stata.com/manuals13/xtxtgee.pdf
+"""
+
 from statsmodels.compat.python import iterkeys, itervalues, zip, range
 from statsmodels.stats.correlation_tools import cov_nearest
 import numpy as np
+import pandas as pd
 from scipy import linalg as spl
-from statsmodels.tools.sm_exceptions import (ConvergenceWarning,
-                                             IterationLimitWarning)
+from collections import defaultdict
+from statsmodels.tools.sm_exceptions import (ConvergenceWarning, OutputWarning,
+                                             NotImplementedWarning)
 import warnings
 
 
 class CovStruct(object):
     """
-    A base class for correlation and covariance structures of grouped
-    data.
+    Base class for correlation and covariance structures.
 
-    Each implementation of this class takes the residuals from a
-    regression model that has been fitted to grouped data, and uses
+    An implementation of this class takes the residuals from a
+    regression model that has been fit to grouped data, and uses
     them to estimate the within-group dependence structure of the
     random errors in the model.
 
-    The state of the covariance structure is represented through the
-    value of the class variable `dep_params`.  The default state of a
-    newly-created instance should correspond to the identity
-    correlation matrix.
+    The current state of the covariance structure is represented
+    through the value of the `dep_params`  attribute.
+
+    The default state of a newly-created instance should always be
+    the identity correlation matrix.
     """
 
     def __init__(self, cov_nearest_method="clipped"):
@@ -32,10 +43,10 @@ class CovStruct(object):
         # adjusted.
         self.cov_adjust = []
 
-        # Method for projecting the covariance matrix if it not SPD.
+        # Method for projecting the covariance matrix if it is not
+        # PSD.
         self.cov_nearest_method = cov_nearest_method
 
-
     def initialize(self, model):
         """
         Called by GEE, used by implementations that need additional
@@ -50,7 +61,7 @@ class CovStruct(object):
 
     def update(self, params):
         """
-        Updates the association parameter values based on the current
+        Update the association parameter values based on the current
         regression coefficients.
 
         Parameters
@@ -120,7 +131,7 @@ class CovStruct(object):
 
         If the covariance matrix is singular or not SPD, it is
         projected to the nearest such matrix.  These projection events
-        are recorded in the fit_history member of the GEE model.
+        are recorded in the fit_history attribute of the GEE model.
 
         Systems of linear equations with the covariance matrix as the
         left hand side (LHS) are solved for different right hand sides
@@ -132,7 +143,6 @@ class CovStruct(object):
         """
 
         vmat, is_cor = self.covariance_matrix(expval, index)
-
         if is_cor:
             vmat *= np.outer(stdev, stdev)
 
@@ -151,13 +161,17 @@ class CovStruct(object):
                                    threshold=threshold)
                 threshold *= 2
                 cov_adjust += 1
+                msg = "At least one covariance matrix was not PSD "
+                msg += "and required projection."
+                warnings.warn(msg)
 
         self.cov_adjust.append(cov_adjust)
 
         # Last resort if we still can't factor the covariance matrix.
-        if success == False:
-            warnings.warn("Unable to condition covariance matrix to an SPD matrix using cov_nearest",
-                          ConvergenceWarning)
+        if not success:
+            warnings.warn(
+                "Unable to condition covariance matrix to an SPD "
+                "matrix using cov_nearest", ConvergenceWarning)
             vmat = np.diag(np.diag(vmat))
             vco = spl.cho_factor(vmat)
 
@@ -186,7 +200,7 @@ class Independence(CovStruct):
         return np.eye(dim, dtype=np.float64), True
 
     def covariance_matrix_solve(self, expval, index, stdev, rhs):
-        v = stdev**2
+        v = stdev ** 2
         rslt = []
         for x in rhs:
             if x.ndim == 1:
@@ -200,7 +214,8 @@ class Independence(CovStruct):
     covariance_matrix_solve.__doc__ = CovStruct.covariance_matrix_solve.__doc__
 
     def summary(self):
-        return "Observations within a cluster are modeled as being independent."
+        return ("Observations within a cluster are modeled "
+                "as being independent.")
 
 
 class Exchangeable(CovStruct):
@@ -220,33 +235,43 @@ class Exchangeable(CovStruct):
         endog = self.model.endog_li
 
         nobs = self.model.nobs
-        dim = len(params)
 
         varfunc = self.model.family.variance
 
         cached_means = self.model.cached_means
 
-        residsq_sum, scale, nterm = 0, 0, 0
-        for i in range(self.model.num_group):
+        has_weights = self.model.weights is not None
+        weights_li = self.model.weights
 
+        residsq_sum, scale = 0, 0
+        fsum1, fsum2, n_pairs = 0., 0., 0.
+        for i in range(self.model.num_group):
             expval, _ = cached_means[i]
             stdev = np.sqrt(varfunc(expval))
             resid = (endog[i] - expval) / stdev
+            f = weights_li[i] if has_weights else 1.
 
+            ssr = np.sum(resid * resid)
+            scale += f * ssr
+            fsum1 += f * len(endog[i])
+
+            residsq_sum += f * (resid.sum() ** 2 - ssr) / 2
             ngrp = len(resid)
-            residsq = np.outer(resid, resid)
-            scale += np.trace(residsq)
-            residsq = np.tril(residsq, -1)
-            residsq_sum += residsq.sum()
-            nterm += 0.5 * ngrp * (ngrp - 1)
+            npr = 0.5 * ngrp * (ngrp - 1)
+            fsum2 += f * npr
+            n_pairs += npr
 
-        scale /= (nobs - dim)
-        self.dep_params = residsq_sum / (scale * (nterm - dim))
+        ddof = self.model.ddof_scale
+        scale /= (fsum1 * (nobs - ddof) / float(nobs))
+        residsq_sum /= scale
+        self.dep_params = residsq_sum / \
+            (fsum2 * (n_pairs - ddof) / float(n_pairs))
 
     def covariance_matrix(self, expval, index):
         dim = len(expval)
         dp = self.dep_params * np.ones((dim, dim), dtype=np.float64)
-        return  dp + (1. - self.dep_params) * np.eye(dim), True
+        np.fill_diagonal(dp, 1)
+        return dp, True
 
     def covariance_matrix_solve(self, expval, index, stdev, rhs):
 
@@ -284,8 +309,8 @@ class Nested(CovStruct):
     A nested working dependence structure.
 
     A working dependence structure that captures a nested hierarchy of
-    groups, each level of which contributes to the random error term
-    of the model.
+    groups.  Each level of grouping contributes to the random error
+    structure of the model.
 
     When using this working covariance structure, `dep_data` of the
     GEE instance should contain a n_obs x k matrix of 0/1 indicators,
@@ -326,7 +351,7 @@ class Nested(CovStruct):
 
     The variance components are estimated using least squares
     regression of the products r*r', for standardized residuals r and
-    r' in the same group, on a vector of indicators defining which
+    r' in the same group, on a matrix of indicators defining which
     variance components are shared by r and r'.
     """
 
@@ -346,10 +371,15 @@ class Nested(CovStruct):
 
         super(Nested, self).initialize(model)
 
+        if self.model.weights is not None:
+            warnings.warn("weights not implemented for nested cov_struct, "
+                          "using unweighted covariance estimate",
+                          NotImplementedWarning)
+
         # A bit of processing of the nest data
         id_matrix = np.asarray(self.model.dep_data)
         if id_matrix.ndim == 1:
-            id_matrix = id_matrix[:,None]
+            id_matrix = id_matrix[:, None]
         self.id_matrix = id_matrix
 
         endog = self.model.endog_li
@@ -372,16 +402,16 @@ class Nested(CovStruct):
             # This is used to construct the working correlation
             # matrix.
             ilabel = np.zeros((ngrp, ngrp), dtype=np.int32)
-            ilabel[[ix1, ix2]] = ncm + 1
-            ilabel[[ix2, ix1]] = ncm + 1
+            ilabel[(ix1, ix2)] = ncm + 1
+            ilabel[(ix2, ix1)] = ncm + 1
             ilabels.append(ilabel)
 
             # This is used to estimate the variance components.
-            dsx = np.zeros((len(ix1), n_nest+1), dtype=np.float64)
-            dsx[:,0] = 1
+            dsx = np.zeros((len(ix1), n_nest + 1), dtype=np.float64)
+            dsx[:, 0] = 1
             for k in np.unique(ncm):
                 ii = np.flatnonzero(ncm == k)
-                dsx[ii, 1:k+1] = 1
+                dsx[ii, 1:k + 1] = 1
             designx.append(dsx)
 
         self.designx = np.concatenate(designx, axis=0)
@@ -395,7 +425,6 @@ class Nested(CovStruct):
     def update(self, params):
 
         endog = self.model.endog_li
-        offset = self.model.offset_li
 
         nobs = self.model.nobs
         dim = len(params)
@@ -414,12 +443,12 @@ class Nested(CovStruct):
             expval, _ = cached_means[i]
 
             stdev = np.sqrt(varfunc(expval))
-            resid = (endog[i] - offset[i] - expval) / stdev
+            resid = (endog[i] - expval) / stdev
 
             ix1, ix2 = np.tril_indices(len(resid), -1)
             dvmat.append(resid[ix1] * resid[ix2])
 
-            scale += np.sum(resid**2)
+            scale += np.sum(resid ** 2)
 
         dvmat = np.concatenate(dvmat)
         scale /= (nobs - dim)
@@ -427,7 +456,7 @@ class Nested(CovStruct):
         # Use least squares regression to estimate the variance
         # components
         vcomp_coeff = np.dot(self.designx_v, np.dot(self.designx_u.T,
-                                dvmat) / self.designx_s)
+                                                    dvmat) / self.designx_s)
 
         self.vcomp_coeff = np.clip(vcomp_coeff, 0, np.inf)
         self.scale = scale
@@ -458,24 +487,186 @@ class Nested(CovStruct):
         dependence structure.
         """
 
-        msg = "Variance estimates\n------------------\n"
-        for k in range(len(self.vcomp_coeff)):
-            msg += "Component %d: %.3f\n" % (k+1, self.vcomp_coeff[k])
-        msg += "Residual: %.3f\n" % (self.scale -
-                                     np.sum(self.vcomp_coeff))
-        return msg
+        dep_names = ["Groups"]
+        if hasattr(self.model, "_dep_data_names"):
+            dep_names.extend(self.model._dep_data_names)
+        else:
+            dep_names.extend(["Component %d:" % (k + 1) for k in range(len(self.vcomp_coeff) - 1)])
+        if hasattr(self.model, "_groups_name"):
+            dep_names[0] = self.model._groups_name
+        dep_names.append("Residual")
 
+        vc = self.vcomp_coeff.tolist()
+        vc.append(self.scale - np.sum(vc))
+
+        smry = pd.DataFrame({"Variance": vc}, index=dep_names)
+
+        return smry
+
+
+class Stationary(CovStruct):
+    """
+    A stationary covariance structure.
+
+    The correlation between two observations is an arbitrary function
+    of the distance between them.  Distances up to a given maximum
+    value are included in the covariance model.
+
+    Parameters
+    ----------
+    max_lag : float
+        The largest distance that is included in the covariance model.
+    grid : bool
+        If True, the index positions in the data (after dropping missing
+        values) are used to define distances, and the `time` variable is
+        ignored.
+    """
+
+    def __init__(self, max_lag=1, grid=False):
+
+        super(Stationary, self).__init__()
+        self.max_lag = max_lag
+        self.grid = grid
+        self.dep_params = np.zeros(max_lag + 1)
+
+    def initialize(self, model):
+
+        super(Stationary, self).initialize(model)
+
+        # Time used as an index needs to be integer type.
+        if not self.grid:
+            time = self.model.time[:, 0].astype(np.int32)
+            self.time = self.model.cluster_list(time)
+
+    def update(self, params):
+
+        if self.grid:
+            self.update_grid(params)
+        else:
+            self.update_nogrid(params)
+
+    def update_grid(self, params):
+
+        endog = self.model.endog_li
+        cached_means = self.model.cached_means
+        varfunc = self.model.family.variance
+
+        dep_params = np.zeros(self.max_lag + 1)
+        for i in range(self.model.num_group):
+
+            expval, _ = cached_means[i]
+            stdev = np.sqrt(varfunc(expval))
+            resid = (endog[i] - expval) / stdev
+
+            dep_params[0] += np.sum(resid * resid) / len(resid)
+            for j in range(1, self.max_lag + 1):
+                v = resid[j:]
+                dep_params[j] += np.sum(resid[0:-j] * v) / len(v)
+
+        dep_params /= dep_params[0]
+        self.dep_params = dep_params
+
+    def update_nogrid(self, params):
+
+        endog = self.model.endog_li
+        cached_means = self.model.cached_means
+        varfunc = self.model.family.variance
+
+        dep_params = np.zeros(self.max_lag + 1)
+        dn = np.zeros(self.max_lag + 1)
+        resid_ssq = 0
+        resid_ssq_n = 0
+        for i in range(self.model.num_group):
+
+            expval, _ = cached_means[i]
+            stdev = np.sqrt(varfunc(expval))
+            resid = (endog[i] - expval) / stdev
+
+            j1, j2 = np.tril_indices(len(expval), -1)
+            dx = np.abs(self.time[i][j1] - self.time[i][j2])
+            ii = np.flatnonzero(dx <= self.max_lag)
+            j1 = j1[ii]
+            j2 = j2[ii]
+            dx = dx[ii]
+
+            vs = np.bincount(dx, weights=resid[j1] * resid[j2],
+                             minlength=self.max_lag + 1)
+            vd = np.bincount(dx, minlength=self.max_lag + 1)
+
+            resid_ssq += np.sum(resid**2)
+            resid_ssq_n += len(resid)
+
+            ii = np.flatnonzero(vd > 0)
+            if len(ii) > 0:
+                dn[ii] += 1
+                dep_params[ii] += vs[ii] / vd[ii]
+
+        i0 = np.flatnonzero(dn > 0)
+        dep_params[i0] /= dn[i0]
+        resid_msq = resid_ssq / resid_ssq_n
+        dep_params /= resid_msq
+        self.dep_params = dep_params
+
+    def covariance_matrix(self, endog_expval, index):
+
+        if self.grid:
+            return self.covariance_matrix_grid(endog_expval, index)
+
+        j1, j2 = np.tril_indices(len(endog_expval), -1)
+        dx = np.abs(self.time[index][j1] - self.time[index][j2])
+        ii = np.flatnonzero(dx <= self.max_lag)
+        j1 = j1[ii]
+        j2 = j2[ii]
+        dx = dx[ii]
+
+        cmat = np.eye(len(endog_expval))
+        cmat[j1, j2] = self.dep_params[dx]
+        cmat[j2, j1] = self.dep_params[dx]
+
+        return cmat, True
+
+    def covariance_matrix_grid(self, endog_expval, index):
+
+        from scipy.linalg import toeplitz
+        r = np.zeros(len(endog_expval))
+        r[0] = 1
+        r[1:self.max_lag + 1] = self.dep_params[1:]
+        return toeplitz(r), True
+
+    def covariance_matrix_solve(self, expval, index, stdev, rhs):
+
+        if not self.grid:
+            return super(Stationary, self).covariance_matrix_solve(
+                expval, index, stdev, rhs)
+
+        from statsmodels.tools.linalg import stationary_solve
+        r = np.zeros(len(expval))
+        r[0:self.max_lag] = self.dep_params[1:]
+        return [stationary_solve(r, x) for x in rhs]
+
+    update.__doc__ = CovStruct.update.__doc__
+    covariance_matrix.__doc__ = CovStruct.covariance_matrix.__doc__
+    covariance_matrix_solve.__doc__ = CovStruct.covariance_matrix_solve.__doc__
+
+    def summary(self):
+
+        lag = np.arange(self.max_lag + 1)
+        return pd.DataFrame({"Lag": lag, "Cov": self.dep_params})
 
 
 class Autoregressive(CovStruct):
     """
-    An autoregressive working dependence structure.
+    A first-order autoregressive working dependence structure.
 
     The dependence is defined in terms of the `time` component of the
-    parent GEE class.  Time represents a potentially multidimensional
+    parent GEE class, which defaults to the index position of each
+    value within its cluster, based on the order of values in the
+    input data set.  Time represents a potentially multidimensional
     index from which distances between pairs of observations can be
-    determined.  The correlation between two observations in the same
-    cluster is dep_params^distance, where `dep_params` is the
+    determined.
+
+    The correlation between two observations in the same cluster is
+    dep_params^distance, where `dep_params` contains the (scalar)
     autocorrelation parameter to be estimated, and `distance` is the
     distance between the two observations, calculated from their
     corresponding time values.  `time` is stored as an n_obs x k
@@ -489,8 +680,8 @@ class Autoregressive(CovStruct):
     Parameters
     ----------
     dist_func: function from R^k x R^k to R^+, optional
-       A function that computes the distance between the two
-       observations based on their `time` values.
+        A function that computes the distance between the two
+        observations based on their `time` values.
 
     References
     ----------
@@ -516,6 +707,11 @@ class Autoregressive(CovStruct):
 
     def update(self, params):
 
+        if self.model.weights is not None:
+            warnings.warn("weights not implemented for autoregressive "
+                          "cov_struct, using unweighted covariance estimate",
+                          NotImplementedWarning)
+
         endog = self.model.endog_li
         time = self.model.time_li
 
@@ -544,8 +740,8 @@ class Autoregressive(CovStruct):
         cached_means = self.model.cached_means
 
         # Weights
-        var = 1. - self.dep_params**(2*designx)
-        var /= 1. - self.dep_params**2
+        var = 1. - self.dep_params ** (2 * designx)
+        var /= 1. - self.dep_params ** 2
         wts = 1. / var
         wts /= wts.sum()
 
@@ -565,8 +761,8 @@ class Autoregressive(CovStruct):
 
         # Need to minimize this
         def fitfunc(a):
-            dif = residmat[:, 0] - (a**designx)*residmat[:, 1]
-            return np.dot(dif**2, wts)
+            dif = residmat[:, 0] - (a ** designx) * residmat[:, 1]
+            return np.dot(dif ** 2, wts)
 
         # Left bracket point
         b_lft, f_lft = 0., fitfunc(0.)
@@ -597,7 +793,7 @@ class Autoregressive(CovStruct):
         if self.dep_params == 0:
             return np.eye(ngrp, dtype=np.float64), True
         idx = np.arange(ngrp)
-        cmat = self.dep_params**np.abs(idx[:, None] - idx[None, :])
+        cmat = self.dep_params ** np.abs(idx[:, None] - idx[None, :])
         return cmat, True
 
     def covariance_matrix_solve(self, expval, index, stdev, rhs):
@@ -608,12 +804,12 @@ class Autoregressive(CovStruct):
 
         # LHS has 1 column
         if k == 1:
-            return [x / stdev**2 for x in rhs]
+            return [x / stdev ** 2 for x in rhs]
 
         # LHS has 2 columns
         if k == 2:
             mat = np.array([[1, -self.dep_params], [-self.dep_params, 1]])
-            mat /= (1. - self.dep_params**2)
+            mat /= (1. - self.dep_params ** 2)
             for x in rhs:
                 if x.ndim == 1:
                     x1 = x / stdev
@@ -631,9 +827,9 @@ class Autoregressive(CovStruct):
         # the inverse.  c0 is on the diagonal, except for the first
         # and last position.  c1 is on the first and last position of
         # the diagonal.  c2 is on the sub/super diagonal.
-        c0 = (1. + self.dep_params**2) / (1. - self.dep_params**2)
-        c1 = 1. / (1. - self.dep_params**2)
-        c2 = -self.dep_params / (1. - self.dep_params**2)
+        c0 = (1. + self.dep_params ** 2) / (1. - self.dep_params ** 2)
+        c1 = 1. / (1. - self.dep_params ** 2)
+        c2 = -self.dep_params / (1. - self.dep_params ** 2)
         soln = []
         for x in rhs:
             flatten = False
@@ -643,12 +839,12 @@ class Autoregressive(CovStruct):
             x1 = x / stdev[:, None]
 
             z0 = np.zeros((1, x.shape[1]))
-            rhs1 = np.concatenate((x[1:,:], z0), axis=0)
-            rhs2 = np.concatenate((z0, x[0:-1,:]), axis=0)
+            rhs1 = np.concatenate((x[1:, :], z0), axis=0)
+            rhs2 = np.concatenate((z0, x[0:-1, :]), axis=0)
 
-            y = c0*x + c2*rhs1 + c2*rhs2
-            y[0, :] = c1*x[0, :] + c2*x[1, :]
-            y[-1, :] = c1*x[-1, :] + c2*x[-2, :]
+            y = c0 * x + c2 * rhs1 + c2 * rhs2
+            y[0, :] = c1 * x[0, :] + c2 * x[1, :]
+            y[-1, :] = c1 * x[-1, :] + c2 * x[-2, :]
 
             y /= stdev[:, None]
 
@@ -669,7 +865,41 @@ class Autoregressive(CovStruct):
                 self.dep_params)
 
 
-class GlobalOddsRatio(CovStruct):
+class CategoricalCovStruct(CovStruct):
+    """
+    Parent class for covariance structure for categorical data models.
+
+    Attributes
+    ----------
+    nlevel : int
+        The number of distinct levels for the outcome variable.
+    ibd : list
+        A list whose i^th element ibd[i] is an array whose rows
+        contain integer pairs (a,b), where endog_li[i][a:b] is the
+        subvector of binary indicators derived from the same ordinal
+        value.
+    """
+
+    def initialize(self, model):
+
+        super(CategoricalCovStruct, self).initialize(model)
+
+        self.nlevel = len(model.endog_values)
+        self._ncut = self.nlevel - 1
+
+        from numpy.lib.stride_tricks import as_strided
+        b = np.dtype(np.int64).itemsize
+
+        ibd = []
+        for v in model.endog_li:
+            jj = np.arange(0, len(v) + 1, self._ncut, dtype=np.int64)
+            jj = as_strided(jj, shape=(len(jj) - 1, 2), strides=(b, b))
+            ibd.append(jj)
+
+        self.ibd = ibd
+
+
+class GlobalOddsRatio(CategoricalCovStruct):
     """
     Estimate the global odds ratio for a GEE with ordinal or nominal
     data.
@@ -707,47 +937,35 @@ class GlobalOddsRatio(CovStruct):
 
         super(GlobalOddsRatio, self).initialize(model)
 
-        self.nlevel = len(model.endog_values)
-        self.ncut = self.nlevel - 1
-
-        ibd = []
-        for v in model.endog_li:
-            jj = np.arange(0, len(v) + 1, self.ncut)
-            ibd1 = np.hstack((jj[0:-1][:, None], jj[1:][:, None]))
-            ibd1 = [(jj[k], jj[k + 1]) for k in range(len(jj) - 1)]
-            ibd.append(ibd1)
-        self.ibd = ibd
+        if self.model.weights is not None:
+            warnings.warn("weights not implemented for GlobalOddsRatio "
+                          "cov_struct, using unweighted covariance estimate",
+                          NotImplementedWarning)
 
         # Need to restrict to between-subject pairs
         cpp = []
         for v in model.endog_li:
 
             # Number of subjects in this group
-            m = int(len(v) / self.ncut)
+            m = int(len(v) / self._ncut)
+            i1, i2 = np.tril_indices(m, -1)
 
             cpp1 = {}
-            # Loop over distinct subject pairs
-            for i1 in range(m):
-                for i2 in range(i1):
-                    # Loop over cut point pairs
-                    for k1 in range(self.ncut):
-                        for k2 in range(k1+1):
-                            if (k2, k1) not in cpp1:
-                                cpp1[(k2, k1)] = []
-                            j1 = i1*self.ncut + k1
-                            j2 = i2*self.ncut + k2
-                            cpp1[(k2, k1)].append([j2, j1])
+            for k1 in range(self._ncut):
+                for k2 in range(k1 + 1):
+                    jj = np.zeros((len(i1), 2), dtype=np.int64)
+                    jj[:, 0] = i1 * self._ncut + k1
+                    jj[:, 1] = i2 * self._ncut + k2
+                    cpp1[(k2, k1)] = jj
 
-            for k in cpp1.keys():
-                cpp1[k] = np.asarray(cpp1[k])
             cpp.append(cpp1)
 
         self.cpp = cpp
 
         # Initialize the dependence parameters
         self.crude_or = self.observed_crude_oddsratio()
-        self.dep_params = self.crude_or
-
+        if self.model.update_dep:
+            self.dep_params = self.crude_or
 
     def pooled_odds_ratio(self, tables):
         """
@@ -764,7 +982,7 @@ class GlobalOddsRatio(CovStruct):
         log_oddsratio, var = [], []
         for table in tables:
             lor = np.log(table[1, 1]) + np.log(table[0, 0]) -\
-                  np.log(table[0, 1]) - np.log(table[1, 0])
+                np.log(table[0, 1]) - np.log(table[1, 0])
             log_oddsratio.append(lor)
             var.append((1 / table.astype(np.float64)).sum())
 
@@ -772,7 +990,7 @@ class GlobalOddsRatio(CovStruct):
         wts = [1 / v for v in var]
         wtsum = sum(wts)
         wts = [w / wtsum for w in wts]
-        log_pooled_or = sum([w*e for w, e in zip(wts, log_oddsratio)])
+        log_pooled_or = sum([w * e for w, e in zip(wts, log_oddsratio)])
 
         return np.exp(log_pooled_or)
 
@@ -836,19 +1054,17 @@ class GlobalOddsRatio(CovStruct):
         else:
             psum = endog_expval[:, None] + endog_expval[None, :]
             pprod = endog_expval[:, None] * endog_expval[None, :]
-            pfac = np.sqrt((1. + psum * (current_or - 1.))**2 +
+            pfac = np.sqrt((1. + psum * (current_or - 1.)) ** 2 +
                            4 * current_or * (1. - current_or) * pprod)
-            vmat = 1. +  psum * (current_or - 1.) - pfac
+            vmat = 1. + psum * (current_or - 1.) - pfac
             vmat /= 2. * (current_or - 1)
 
         # Fix E[YY'] for elements that belong to same observation
         for bdl in ibd:
             evy = endog_expval[bdl[0]:bdl[1]]
             if self.endog_type == "ordinal":
-                eyr = np.outer(evy, np.ones(len(evy)))
-                eyc = np.outer(np.ones(len(evy)), evy)
-                vmat[bdl[0]:bdl[1], bdl[0]:bdl[1]] = \
-                    np.where(eyr < eyc, eyr, eyc)
+                vmat[bdl[0]:bdl[1], bdl[0]:bdl[1]] =\
+                    np.minimum.outer(evy, evy)
             else:
                 vmat[bdl[0]:bdl[1], bdl[0]:bdl[1]] = np.diag(evy)
 
@@ -860,7 +1076,6 @@ class GlobalOddsRatio(CovStruct):
         params.
         """
 
-        endog = self.model.endog_li
         cpp = self.cpp
         cached_means = self.model.cached_means
 
@@ -898,10 +1113,326 @@ class GlobalOddsRatio(CovStruct):
             warnings.warn("dep_params became inf, resetting to 1",
                           ConvergenceWarning)
 
-
     update.__doc__ = CovStruct.update.__doc__
     covariance_matrix.__doc__ = CovStruct.covariance_matrix.__doc__
 
     def summary(self):
 
         return "Global odds ratio: %.3f\n" % self.dep_params
+
+
+class OrdinalIndependence(CategoricalCovStruct):
+    """
+    An independence covariance structure for ordinal models.
+
+    The working covariance between indicators derived from different
+    observations is zero.  The working covariance between indicators
+    derived form a common observation is determined from their current
+    mean values.
+
+    There are no parameters to estimate in this covariance structure.
+    """
+
+    def covariance_matrix(self, expected_value, index):
+
+        ibd = self.ibd[index]
+        n = len(expected_value)
+        vmat = np.zeros((n, n))
+
+        for bdl in ibd:
+            ev = expected_value[bdl[0]:bdl[1]]
+            vmat[bdl[0]:bdl[1], bdl[0]:bdl[1]] =\
+                np.minimum.outer(ev, ev) - np.outer(ev, ev)
+
+        return vmat, False
+
+    # Nothing to update
+    def update(self, params):
+        pass
+
+
+class NominalIndependence(CategoricalCovStruct):
+    """
+    An independence covariance structure for nominal models.
+
+    The working covariance between indicators derived from different
+    observations is zero.  The working covariance between indicators
+    derived form a common observation is determined from their current
+    mean values.
+
+    There are no parameters to estimate in this covariance structure.
+    """
+
+    def covariance_matrix(self, expected_value, index):
+
+        ibd = self.ibd[index]
+        n = len(expected_value)
+        vmat = np.zeros((n, n))
+
+        for bdl in ibd:
+            ev = expected_value[bdl[0]:bdl[1]]
+            vmat[bdl[0]:bdl[1], bdl[0]:bdl[1]] =\
+                np.diag(ev) - np.outer(ev, ev)
+
+        return vmat, False
+
+    # Nothing to update
+    def update(self, params):
+        pass
+
+
+class Equivalence(CovStruct):
+    """
+    A covariance structure defined in terms of equivalence classes.
+
+    An 'equivalence class' is a set of pairs of observations such that
+    the covariance of every pair within the equivalence class has a
+    common value.
+
+    Parameters
+    ----------
+    pairs : dict-like
+      A dictionary of dictionaries, where `pairs[group][label]`
+      provides the indices of all pairs of observations in the group
+      that have the same covariance value.  Specifically,
+      `pairs[group][label]` is a tuple `(j1, j2)`, where `j1` and `j2`
+      are integer arrays of the same length.  `j1[i], j2[i]` is one
+      index pair that belongs to the `label` equivalence class.  Only
+      one triangle of each covariance matrix should be included.
+      Positions where j1 and j2 have the same value are variance
+      parameters.
+    labels : array-like
+      An array of labels such that every distinct pair of labels
+      defines an equivalence class.  Either `labels` or `pairs` must
+      be provided.  When the two labels in a pair are equal two
+      equivalence classes are defined: one for the diagonal elements
+      (corresponding to variances) and one for the off-diagonal
+      elements (corresponding to covariances).
+    return_cov : boolean
+      If True, `covariance_matrix` returns an estimate of the
+      covariance matrix, otherwise returns an estimate of the
+      correlation matrix.
+
+    Notes
+    -----
+    Using `labels` to define the class is much easier than using
+    `pairs`, but is less general.
+
+    Any pair of values not contained in `pairs` will be assigned zero
+    covariance.
+
+    The index values in `pairs` are row indices into the `exog`
+    matrix.  They are not updated if missing data are present.  When
+    using this covariance structure, missing data should be removed
+    before constructing the model.
+
+    If using `labels`, after a model is defined using the covariance
+    structure it is possible to remove a label pair from the second
+    level of the `pairs` dictionary to force the corresponding
+    covariance to be zero.
+
+    Examples
+    --------
+    The following sets up the `pairs` dictionary for a model with two
+    groups, equal variance for all observations, and constant
+    covariance for all pairs of observations within each group.
+
+    >> pairs = {0: {}, 1: {}}
+    >> pairs[0][0] = (np.r_[0, 1, 2], np.r_[0, 1, 2])
+    >> pairs[0][1] = np.tril_indices(3, -1)
+    >> pairs[1][0] = (np.r_[3, 4, 5], np.r_[3, 4, 5])
+    >> pairs[1][2] = 3 + np.tril_indices(3, -1)
+    """
+
+    def __init__(self, pairs=None, labels=None, return_cov=False):
+
+        super(Equivalence, self).__init__()
+
+        if (pairs is None) and (labels is None):
+            raise ValueError(
+                "Equivalence cov_struct requires either `pairs` or `labels`")
+
+        if (pairs is not None) and (labels is not None):
+            raise ValueError(
+                "Equivalence cov_struct accepts only one of `pairs` "
+                "and `labels`")
+
+        if pairs is not None:
+            import copy
+            self.pairs = copy.deepcopy(pairs)
+
+        if labels is not None:
+            self.labels = np.asarray(labels)
+
+        self.return_cov = return_cov
+
+    def _make_pairs(self, i, j):
+        """
+        Create arrays containing all unique ordered pairs of i, j.
+
+        The arrays i and j must be one-dimensional containing non-negative
+        integers.
+        """
+
+        mat = np.zeros((len(i) * len(j), 2), dtype=np.int32)
+
+        # Create the pairs and order them
+        f = np.ones(len(j))
+        mat[:, 0] = np.kron(f, i).astype(np.int32)
+        f = np.ones(len(i))
+        mat[:, 1] = np.kron(j, f).astype(np.int32)
+        mat.sort(1)
+
+        # Remove repeated rows
+        try:
+            dtype = np.dtype((np.void, mat.dtype.itemsize * mat.shape[1]))
+            bmat = np.ascontiguousarray(mat).view(dtype)
+            _, idx = np.unique(bmat, return_index=True)
+        except TypeError:
+            # workaround for old numpy that can't call unique with complex
+            # dtypes
+            rs = np.random.RandomState(4234)
+            bmat = np.dot(mat, rs.uniform(size=mat.shape[1]))
+            _, idx = np.unique(bmat, return_index=True)
+        mat = mat[idx, :]
+
+        return mat[:, 0], mat[:, 1]
+
+    def _pairs_from_labels(self):
+
+        from collections import defaultdict
+        pairs = defaultdict(lambda: defaultdict(lambda: None))
+
+        model = self.model
+
+        df = pd.DataFrame({"labels": self.labels, "groups": model.groups})
+        gb = df.groupby(["groups", "labels"])
+
+        ulabels = np.unique(self.labels)
+
+        for g_ix, g_lb in enumerate(model.group_labels):
+
+            # Loop over label pairs
+            for lx1 in range(len(ulabels)):
+                for lx2 in range(lx1 + 1):
+
+                    lb1 = ulabels[lx1]
+                    lb2 = ulabels[lx2]
+
+                    try:
+                        i1 = gb.groups[(g_lb, lb1)]
+                        i2 = gb.groups[(g_lb, lb2)]
+                    except KeyError:
+                        continue
+
+                    i1, i2 = self._make_pairs(i1, i2)
+
+                    clabel = str(lb1) + "/" + str(lb2)
+
+                    # Variance parameters belong in their own equiv class.
+                    jj = np.flatnonzero(i1 == i2)
+                    if len(jj) > 0:
+                        clabelv = clabel + "/v"
+                        pairs[g_lb][clabelv] = (i1[jj], i2[jj])
+
+                    # Covariance parameters
+                    jj = np.flatnonzero(i1 != i2)
+                    if len(jj) > 0:
+                        i1 = i1[jj]
+                        i2 = i2[jj]
+                        pairs[g_lb][clabel] = (i1, i2)
+
+        self.pairs = pairs
+
+    def initialize(self, model):
+
+        super(Equivalence, self).initialize(model)
+
+        if self.model.weights is not None:
+            warnings.warn("weights not implemented for equalence cov_struct, "
+                          "using unweighted covariance estimate",
+                          NotImplementedWarning)
+
+        if not hasattr(self, 'pairs'):
+            self._pairs_from_labels()
+
+        # Initialize so that any equivalence class containing a
+        # variance parameter has value 1.
+        self.dep_params = defaultdict(lambda: 0.)
+        self._var_classes = set([])
+        for gp in self.model.group_labels:
+            for lb in self.pairs[gp]:
+                j1, j2 = self.pairs[gp][lb]
+                if np.any(j1 == j2):
+                    if not np.all(j1 == j2):
+                        warnings.warn(
+                            "equivalence class contains both variance "
+                            "and covariance parameters", OutputWarning)
+                    self._var_classes.add(lb)
+                    self.dep_params[lb] = 1
+
+        # Need to start indexing at 0 within each group.
+        # rx maps olds indices to new indices
+        rx = -1 * np.ones(len(self.model.endog), dtype=np.int32)
+        for g_ix, g_lb in enumerate(self.model.group_labels):
+            ii = self.model.group_indices[g_lb]
+            rx[ii] = np.arange(len(ii), dtype=np.int32)
+
+        # Reindex
+        for gp in self.model.group_labels:
+            for lb in self.pairs[gp].keys():
+                a, b = self.pairs[gp][lb]
+                self.pairs[gp][lb] = (rx[a], rx[b])
+
+    def update(self, params):
+
+        endog = self.model.endog_li
+        varfunc = self.model.family.variance
+        cached_means = self.model.cached_means
+        dep_params = defaultdict(lambda: [0., 0., 0.])
+        n_pairs = defaultdict(lambda: 0)
+        dim = len(params)
+
+        for k, gp in enumerate(self.model.group_labels):
+            expval, _ = cached_means[k]
+            stdev = np.sqrt(varfunc(expval))
+            resid = (endog[k] - expval) / stdev
+            for lb in self.pairs[gp].keys():
+                if (not self.return_cov) and lb in self._var_classes:
+                    continue
+                jj = self.pairs[gp][lb]
+                dep_params[lb][0] += np.sum(resid[jj[0]] * resid[jj[1]])
+                if not self.return_cov:
+                    dep_params[lb][1] += np.sum(resid[jj[0]] ** 2)
+                    dep_params[lb][2] += np.sum(resid[jj[1]] ** 2)
+                n_pairs[lb] += len(jj[0])
+
+        if self.return_cov:
+            for lb in dep_params.keys():
+                dep_params[lb] = dep_params[lb][0] / (n_pairs[lb] - dim)
+        else:
+            for lb in dep_params.keys():
+                den = np.sqrt(dep_params[lb][1] * dep_params[lb][2])
+                dep_params[lb] = dep_params[lb][0] / den
+            for lb in self._var_classes:
+                dep_params[lb] = 1.
+
+        self.dep_params = dep_params
+        self.n_pairs = n_pairs
+
+    def covariance_matrix(self, expval, index):
+        dim = len(expval)
+        cmat = np.zeros((dim, dim))
+        g_lb = self.model.group_labels[index]
+
+        for lb in self.pairs[g_lb].keys():
+            j1, j2 = self.pairs[g_lb][lb]
+            cmat[j1, j2] = self.dep_params[lb]
+
+        cmat = cmat + cmat.T
+        np.fill_diagonal(cmat, cmat.diagonal() / 2)
+
+        return cmat, not self.return_cov
+
+    update.__doc__ = CovStruct.update.__doc__
+    covariance_matrix.__doc__ = CovStruct.covariance_matrix.__doc__
