@@ -1,7 +1,8 @@
-'''Helper files for pickling'''
+"""Helper files for pickling"""
 from statsmodels.compat.python import cPickle
 from statsmodels.iolib.openfile import get_file_obj
 
+
 def save_pickle(obj, fname):
     """
     Save the object to file via pickling.
@@ -11,6 +12,7 @@ def save_pickle(obj, fname):
     fname : str
         Filename to pickle to
     """
+
     with get_file_obj(fname, 'wb') as fout:
         cPickle.dump(obj, fout, protocol=-1)
 
@@ -28,6 +30,6 @@ def load_pickle(fname):
     -----
     This method can be used to load *both* models and results.
     """
+
     with get_file_obj(fname, 'rb') as fin:
         return cPickle.load(fin)
-
