@@ -126,17 +126,18 @@ class TestKDEMultivariate(KDETestBase):
 
         # Matches R to 3 decimals; results seem more stable than with R.
         # Can be checked with following code:
-        ## import rpy2.robjects as robjects
-        ## from rpy2.robjects.packages import importr
-        ## NP = importr('np')
-        ## r = robjects.r
-        ## D = {"S1": robjects.FloatVector(c1), "S2":robjects.FloatVector(c2),
-        ##      "S3":robjects.FloatVector(c3), "S4":robjects.FactorVector(o),
-        ##      "S5":robjects.FactorVector(o2)}
-        ## df = robjects.DataFrame(D)
-        ## formula = r('~S1+ordered(S4)+ordered(S5)')
-        ## r_bw = NP.npudensbw(formula, data=df, bwmethod='cv.ls')
+        # import rpy2.robjects as robjects
+        # from rpy2.robjects.packages import importr
+        # NP = importr('np')
+        # r = robjects.r
+        # D = {"S1": robjects.FloatVector(c1), "S2":robjects.FloatVector(c2),
+        #      "S3":robjects.FloatVector(c3), "S4":robjects.FactorVector(o),
+        #      "S5":robjects.FactorVector(o2)}
+        # df = robjects.DataFrame(D)
+        # formula = r('~S1+ordered(S4)+ordered(S5)')
+        # r_bw = NP.npudensbw(formula, data=df, bwmethod='cv.ls')
 
+    @pytest.mark.slow
     def test_pdf_mixeddata_LS_vs_ML(self):
         dens_ls = nparam.KDEMultivariate(data=[self.c1, self.o, self.o2],
                                          var_type='coo', bw='cv_ls')
@@ -368,13 +369,13 @@ class TestKDEMultivariateConditional(KDETestBase):
     def test_continuous_cvml_efficient(self):
         nobs = 500
         np.random.seed(12345)
-        O = np.random.binomial(2, 0.5, size=(nobs, ))
+        ovals = np.random.binomial(2, 0.5, size=(nobs, ))
         C1 = np.random.normal(size=(nobs, ))
         noise = np.random.normal(size=(nobs, ))
         b0 = 3
         b1 = 1.2
         b2 = 3.7  # regression coefficients
-        Y = b0+ b1 * C1 + b2*O  + noise
+        Y = b0+ b1 * C1 + b2*ovals  + noise
 
         dens_efficient = nparam.KDEMultivariateConditional(endog=[Y],
             exog=[C1], dep_type='c', indep_type='c', bw='cv_ml',
@@ -398,7 +399,3 @@ class TestKDEMultivariateConditional(KDETestBase):
                                                           randomize=False,
                                                           n_sub=100))
         npt.assert_equal(dens.bw, bw_user)
-
-if __name__ == "__main__":
-    import pytest
-    pytest.main([__file__, '-vvs', '-x', '--pdb'])
