@@ -3,6 +3,7 @@ import numpy as np
 
 #TODO: add plots to weighting functions for online docs.
 
+
 class RobustNorm(object):
     """
     The parent class for the norms used for robust regression.
@@ -27,8 +28,7 @@ class RobustNorm(object):
 
     See Also
     --------
-    statsmodels.rlm for more information on how the estimators are used
-    and the inputs for the methods of RobustNorm and subclasses.
+    statsmodels.rlm
 
     Notes
     -----
@@ -66,7 +66,7 @@ class RobustNorm(object):
         raise NotImplementedError
 
     def psi_deriv(self, z):
-        '''
+        """
         Deriative of psi.  Used to obtain robust covariance matrix.
 
         See statsmodels.rlm for more information.
@@ -74,7 +74,7 @@ class RobustNorm(object):
         Abstract method:
 
         psi_derive = psi'
-        '''
+        """
         raise NotImplementedError
 
     def __call__(self, z):
@@ -83,14 +83,15 @@ class RobustNorm(object):
         """
         return self.rho(z)
 
+
 class LeastSquares(RobustNorm):
 
     """
     Least squares rho for M-estimation and its derived functions.
 
-    See also
+    See Also
     --------
-    statsmodels.robust.norms.RobustNorm for the methods.
+    statsmodels.robust.norms.RobustNorm
     """
 
     def rho(self, z):
@@ -98,7 +99,7 @@ class LeastSquares(RobustNorm):
         The least squares estimator rho function
 
         Parameters
-        -----------
+        ----------
         z : array
             1d array
 
@@ -164,6 +165,7 @@ class LeastSquares(RobustNorm):
         """
         return np.ones(z.shape, np.float64)
 
+
 class HuberT(RobustNorm):
     """
     Huber's T for M estimation.
@@ -174,7 +176,7 @@ class HuberT(RobustNorm):
         The tuning constant for Huber's t function. The default value is
         1.345.
 
-    See also
+    See Also
     --------
     statsmodels.robust.norms.RobustNorm
     """
@@ -190,7 +192,7 @@ class HuberT(RobustNorm):
         return np.less_equal(np.fabs(z), self.t)
 
     def rho(self, z):
-        """
+        r"""
         The robust criterion function for Huber's t.
 
         Parameters
@@ -211,7 +213,7 @@ class HuberT(RobustNorm):
                 (1 - test) * (np.fabs(z) * self.t - 0.5 * self.t**2))
 
     def psi(self, z):
-        """
+        r"""
         The psi function for Huber's t estimator
 
         The analytic derivative of rho
@@ -233,7 +235,7 @@ class HuberT(RobustNorm):
         return test * z + (1 - test) * self.t * np.sign(z)
 
     def weights(self, z):
-        """
+        r"""
         Huber's t weighting function for the IRLS algorithm
 
         The psi function scaled by z
@@ -266,6 +268,7 @@ class HuberT(RobustNorm):
         """
         return np.less_equal(np.fabs(z), self.t)
 
+
 #TODO: untested, but looks right.  RamsayE not available in R or SAS?
 class RamsayE(RobustNorm):
     """
@@ -277,7 +280,7 @@ class RamsayE(RobustNorm):
         The tuning constant for Ramsay's Ea function.  The default value is
         0.3.
 
-    See also
+    See Also
     --------
     statsmodels.robust.norms.RobustNorm
     """
@@ -286,7 +289,7 @@ class RamsayE(RobustNorm):
         self.a = a
 
     def rho(self, z):
-        """
+        r"""
         The robust criterion function for Ramsay's Ea.
 
         Parameters
@@ -304,7 +307,7 @@ class RamsayE(RobustNorm):
                 (1 + self.a * np.fabs(z))) / self.a**2
 
     def psi(self, z):
-        """
+        r"""
         The psi function for Ramsay's Ea estimator
 
         The analytic derivative of rho
@@ -323,7 +326,7 @@ class RamsayE(RobustNorm):
         return z * np.exp(-self.a * np.fabs(z))
 
     def weights(self, z):
-        """
+        r"""
         Ramsay's Ea weighting function for the IRLS algorithm
 
         The psi function scaled by z
@@ -354,6 +357,7 @@ class RamsayE(RobustNorm):
         return np.exp(-self.a * np.fabs(z)) + z**2*\
                 np.exp(-self.a*np.fabs(z))*-self.a/np.fabs(z)
 
+
 class AndrewWave(RobustNorm):
 
     """
@@ -365,12 +369,11 @@ class AndrewWave(RobustNorm):
         The tuning constant for Andrew's Wave function.  The default value is
         1.339.
 
-    See also
+    See Also
     --------
     statsmodels.robust.norms.RobustNorm
-
     """
-    def __init__(self, a = 1.339):
+    def __init__(self, a=1.339):
         self.a = a
 
     def _subset(self, z):
@@ -381,7 +384,7 @@ class AndrewWave(RobustNorm):
         return np.less_equal(np.fabs(z), self.a * np.pi)
 
     def rho(self, z):
-        """
+        r"""
         The robust criterion function for Andrew's wave.
 
         Parameters
@@ -404,7 +407,7 @@ class AndrewWave(RobustNorm):
                 (1 - test) * 2 * a)
 
     def psi(self, z):
-        """
+        r"""
         The psi function for Andrew's wave
 
         The analytic derivative of rho
@@ -428,7 +431,7 @@ class AndrewWave(RobustNorm):
         return test * np.sin(z / a)
 
     def weights(self, z):
-        """
+        r"""
         Andrew's wave weighting function for the IRLS algorithm
 
         The psi function scaled by z
@@ -462,7 +465,8 @@ class AndrewWave(RobustNorm):
         test = self._subset(z)
         return test*np.cos(z / self.a)/self.a
 
-#TODO: this is untested
+
+# TODO: this is untested
 class TrimmedMean(RobustNorm):
     """
     Trimmed mean function for M-estimation.
@@ -473,7 +477,7 @@ class TrimmedMean(RobustNorm):
         The tuning constant for Ramsay's Ea function.  The default value is
         2.0.
 
-    See also
+    See Also
     --------
     statsmodels.robust.norms.RobustNorm
     """
@@ -490,7 +494,7 @@ class TrimmedMean(RobustNorm):
         return np.less_equal(np.fabs(z), self.c)
 
     def rho(self, z):
-        """
+        r"""
         The robust criterion function for least trimmed mean.
 
         Parameters
@@ -511,7 +515,7 @@ class TrimmedMean(RobustNorm):
         return test * z**2 * 0.5
 
     def psi(self, z):
-        """
+        r"""
         The psi function for least trimmed mean
 
         The analytic derivative of rho
@@ -534,7 +538,7 @@ class TrimmedMean(RobustNorm):
         return test * z
 
     def weights(self, z):
-        """
+        r"""
         Least trimmed mean weighting function for the IRLS algorithm
 
         The psi function scaled by z
@@ -567,6 +571,7 @@ class TrimmedMean(RobustNorm):
         test = self._subset(z)
         return test
 
+
 class Hampel(RobustNorm):
     """
 
@@ -580,12 +585,12 @@ class Hampel(RobustNorm):
         The tuning constants for Hampel's function.  The default values are
         a,b,c = 2, 4, 8.
 
-    See also
+    See Also
     --------
     statsmodels.robust.norms.RobustNorm
     """
 
-    def __init__(self, a = 2., b = 4., c = 8.):
+    def __init__(self, a=2., b=4., c=8.):
         self.a = a
         self.b = b
         self.c = c
@@ -601,7 +606,7 @@ class Hampel(RobustNorm):
         return t1, t2, t3
 
     def rho(self, z):
-        """
+        r"""
         The robust criterion function for Hampel's estimator
 
         Parameters
@@ -622,7 +627,9 @@ class Hampel(RobustNorm):
         """
 
         z = np.fabs(z)
-        a = self.a; b = self.b; c = self.c
+        a = self.a
+        b = self.b
+        c = self.c
         t1, t2, t3 = self._subset(z)
         v = (t1 * z**2 * 0.5 +
              t2 * (a * z - a**2 * 0.5) +
@@ -631,7 +638,7 @@ class Hampel(RobustNorm):
         return v
 
     def psi(self, z):
-        """
+        r"""
         The psi function for Hampel's estimator
 
         The analytic derivative of rho
@@ -653,7 +660,9 @@ class Hampel(RobustNorm):
             psi(z) = 0                            for \|z\| > c
         """
         z = np.asarray(z)
-        a = self.a; b = self.b; c = self.c
+        a = self.a
+        b = self.b
+        c = self.c
         t1, t2, t3 = self._subset(z)
         s = np.sign(z)
         z = np.fabs(z)
@@ -663,7 +672,7 @@ class Hampel(RobustNorm):
         return v
 
     def weights(self, z):
-        """
+        r"""
         Hampel weighting function for the IRLS algorithm
 
         The psi function scaled by z
@@ -686,7 +695,9 @@ class Hampel(RobustNorm):
 
         """
         z = np.asarray(z)
-        a = self.a; b = self.b; c = self.c
+        a = self.a
+        b = self.b
+        c = self.c
         t1, t2, t3 = self._subset(z)
         v = (t1 +
             t2 * a/np.fabs(z) +
@@ -698,6 +709,7 @@ class Hampel(RobustNorm):
         t1, t2, t3 = self._subset(z)
         return t1 + t3 * (self.a*np.sign(z)*z)/(np.fabs(z)*(self.c-self.b))
 
+
 class TukeyBiweight(RobustNorm):
     """
 
@@ -725,7 +737,7 @@ class TukeyBiweight(RobustNorm):
         return np.less_equal(z, self.c)
 
     def rho(self, z):
-        """
+        r"""
         The robust criterion function for Tukey's biweight estimator
 
         Parameters
@@ -744,7 +756,7 @@ class TukeyBiweight(RobustNorm):
         return -(1 - (z / self.c)**2)**3 * subset * self.c**2 / 6.
 
     def psi(self, z):
-        """
+        r"""
         The psi function for Tukey's biweight estimator
 
         The analytic derivative of rho
@@ -766,9 +778,8 @@ class TukeyBiweight(RobustNorm):
         subset = self._subset(z)
         return z * (1 - (z / self.c)**2)**2 * subset
 
-
     def weights(self, z):
-        """
+        r"""
         Tukey's biweight weighting function for the IRLS algorithm
 
         The psi function scaled by z
@@ -830,7 +841,7 @@ def estimate_location(a, scale, norm=None, axis=0, initial=None,
         Toleration for convergence.  The default is 1e-06.
 
     Returns
-    --------
+    -------
     mu : array
         Estimate of location
     """
@@ -849,6 +860,5 @@ def estimate_location(a, scale, norm=None, axis=0, initial=None,
             return nmu
         else:
             mu = nmu
-    raise ValueError("location estimator failed to converge in %d iterations"\
-            % maxiter)
-
+    raise ValueError("location estimator failed to converge in %d iterations"
+                     % maxiter)
