@@ -1,14 +1,14 @@
-#collect some imports of verified (at least one example) functions
+# flake8: noqa
+
+# collect some imports of verified (at least one example) functions
 from statsmodels.sandbox.stats.diagnostic import (
     acorr_ljungbox, breaks_cusumolsresid, breaks_hansen, CompareCox, CompareJ,
     compare_cox, compare_j, het_breuschpagan, HetGoldfeldQuandt,
     het_goldfeldquandt, het_arch,
     het_white, recursive_olsresiduals, acorr_breusch_godfrey,
     linear_harvey_collier, linear_rainbow, linear_lm,
-    unitroot_adf,
-    het_breushpagan, acorr_breush_godfrey  # deprecated because of misspelling
-    )
+    spec_white, unitroot_adf)
 
-from ._lilliefors import (kstest_fit, lilliefors, lillifors, kstest_normal,
-                          kstest_exponential)  # lillifors is deprecated
+from ._lilliefors import (kstest_fit, lilliefors, kstest_normal,
+                          kstest_exponential)
 from ._adnorm import normal_ad
