@@ -30,14 +30,14 @@ def _show_versions_only():
     except:
         pass
     try:
-        from statsmodels import version
+        import statsmodels
         has_sm = True
     except ImportError:
         has_sm = False
 
     print('\nStatsmodels\n===========\n')
     if has_sm:
-        print('Installed: %s' % safe_version(version, 'full_version'))
+        print('Installed: %s' % safe_version(statsmodels))
     else:
         print('Not installed')
 
@@ -123,12 +123,6 @@ def _show_versions_only():
     except ImportError:
         print("    pygments: Not installed")
 
-    try:
-        import nose
-        print("nose: %s" % safe_version(nose))
-    except ImportError:
-        print("nose: Not installed")
-
     try:
         import pytest
         print("pytest: %s (%s)" % (safe_version(pytest), dirname(pytest.__file__)))
@@ -162,14 +156,13 @@ def show_versions(show_dirs=True):
 
     try:
         import statsmodels
-        from statsmodels import version
         has_sm = True
     except ImportError:
         has_sm = False
 
     print('\nStatsmodels\n===========\n')
     if has_sm:
-        print('Installed: %s (%s)' % (safe_version(version, 'full_version'),
+        print('Installed: %s (%s)' % (safe_version(statsmodels),
                                       dirname(statsmodels.__file__)))
     else:
         print('Not installed')
@@ -271,12 +264,6 @@ def show_versions(show_dirs=True):
     except ImportError:
         print("    pygments: Not installed")
 
-    try:
-        import nose
-        print("nose: %s (%s)" % (safe_version(nose), dirname(nose.__file__)))
-    except ImportError:
-        print("nose: Not installed")
-
     try:
         import pytest
         print("pytest: %s (%s)" % (safe_version(pytest), dirname(pytest.__file__)))
