@@ -965,11 +965,13 @@ class TestFedFundsConstShort(MarkovRegression):
         expected_marginals[:, 2:] = [[0], [1], [0]]
 
         py_results = markov_switching.py_hamilton_filter(
-            initial_probabilities, regime_transition, conditional_likelihoods)
+            initial_probabilities, regime_transition, conditional_likelihoods,
+            model_order=0)
         assert_allclose(py_results[0], expected_marginals)
 
         cy_results = markov_switching.cy_hamilton_filter(
-            initial_probabilities, regime_transition, conditional_likelihoods)
+            initial_probabilities, regime_transition, conditional_likelihoods,
+            model_order=0)
         assert_allclose(cy_results[0], expected_marginals)
 
     def test_hamilton_filter_order_zero_with_tvtp(self):
@@ -1016,11 +1018,13 @@ class TestFedFundsConstShort(MarkovRegression):
         expected_marginals[:, 6:8] = [[0], [0], [1]]
 
         py_results = markov_switching.py_hamilton_filter(
-            initial_probabilities, regime_transition, conditional_likelihoods)
+            initial_probabilities, regime_transition, conditional_likelihoods,
+            model_order=0)
         assert_allclose(py_results[0], expected_marginals)
 
         cy_results = markov_switching.cy_hamilton_filter(
-            initial_probabilities, regime_transition, conditional_likelihoods)
+            initial_probabilities, regime_transition, conditional_likelihoods,
+            model_order=0)
         assert_allclose(cy_results[0], expected_marginals)
 
     def test_hamilton_filter_shape_checks(self):
@@ -1037,7 +1041,7 @@ class TestFedFundsConstShort(MarkovRegression):
             with assert_raises(ValueError):
                 func(initial_probabilities,
                      regime_transition,
-                     conditional_likelihoods)
+                     conditional_likelihoods, model_order=3)
 
     def test_py_hamilton_filter(self):
         mod = self.model
@@ -1049,9 +1053,11 @@ class TestFedFundsConstShort(MarkovRegression):
         conditional_likelihoods = mod._conditional_likelihoods(params)
 
         actual = markov_switching.py_hamilton_filter(
-            initial_probabilities, regime_transition, conditional_likelihoods)
+            initial_probabilities, regime_transition, conditional_likelihoods,
+            model_order=mod.order)
         desired = markov_switching.cy_hamilton_filter(
-            initial_probabilities, regime_transition, conditional_likelihoods)
+            initial_probabilities, regime_transition, conditional_likelihoods,
+            model_order=mod.order)
 
         for i in range(3):
             assert_allclose(actual[i], desired[i])
@@ -1067,7 +1073,8 @@ class TestFedFundsConstShort(MarkovRegression):
 
         # Hamilton filter
         filter_output = markov_switching.cy_hamilton_filter(
-            initial_probabilities, regime_transition, conditional_likelihoods)
+            initial_probabilities, regime_transition, conditional_likelihoods,
+            model_order=mod.order)
 
         # Kim smoother
         actual = markov_switching.py_kim_smoother(
@@ -1233,3 +1240,31 @@ class TestMumpspcNoconstL1Variance(MarkovRegression):
         super(TestMumpspcNoconstL1Variance, cls).setup_class(
             true, mumpspc[1:], k_regimes=2, trend='nc', exog=mumpspc[:-1],
             switching_variance=True, atol=1e-4)
+
+
+def test_exog_tvtp():
+    exog = np.ones_like(fedfunds)
+
+    mod1 = markov_regression.MarkovRegression(fedfunds, k_regimes=2)
+    mod2 = markov_regression.MarkovRegression(fedfunds, k_regimes=2,
+                                              exog_tvtp=exog)
+
+    params = np.r_[0.98209618, 0.05036498, 3.70877542, 9.55676298, 4.44181911]
+    params_tvtp = params.copy()
+    params_tvtp[0] = mod2._untransform_logistic(
+        np.r_[0.], np.r_[1 - params[0]])
+    params_tvtp[1] = mod2._untransform_logistic(
+        np.r_[0.], np.r_[1 - params[1]])
+
+    res1 = mod1.smooth(params)
+    res2 = mod2.smooth(params_tvtp)
+
+    assert_allclose(res2.llf_obs, res1.llf_obs)
+    assert_allclose(res2.regime_transition - res1.regime_transition, 0,
+                    atol=1e-15)
+    assert_allclose(res2.predicted_joint_probabilities,
+                    res1.predicted_joint_probabilities)
+    assert_allclose(res2.filtered_joint_probabilities,
+                    res1.filtered_joint_probabilities)
+    assert_allclose(res2.smoothed_joint_probabilities,
+                    res1.smoothed_joint_probabilities)
